require 'spec_helper'

require 'tablets/data/processing/filter'

RSpec.describe Tablets::Data::Processing::Filter do
  subject { described_class.new(params, columns) }

  let(:relation) { double }
  let(:params) { { search: { value: value } } }

  describe '#apply' do
    let(:value) { 'query1 query2' }

    before do
      allow_any_instance_of(
        Tablets::Utils::SearchConditionBuilder
      ).to receive(:build) { |builder| "#{builder.column} #{builder.query}" }

      allow(relation).to receive(:where).and_return(relation)
    end

    let(:columns) { [{ search: %w(column1 column2) }] }
    let!(:result) { subject.apply(relation) }

    it 'applies search conditions' do
      expect(relation).to have_received(:where).with(
        '(column1 query1 OR column2 query1) AND ' \
        '(column1 query2 OR column2 query2)'
      )
    end

    it 'returns relation' do
      expect(result).to eq(relation)
    end
  end
end
