require 'spec_helper'

require 'tablets/data/processing/order'

RSpec.describe Tablets::Data::Processing::Order do
  subject { described_class.new(params, columns) }

  let(:relation) { double }
  let(:order) { {} }
  let(:params) { { order: order } }

  describe '#apply' do
    before do
      allow(order).to receive(:values).and_return(values)
      allow(relation).to receive(:order).and_return(relation)
    end

    let!(:result) { subject.apply(relation) }

    context 'simple' do
      let(:field) { 'field' }
      let(:direction) { 'DESC' }
      let(:nulls) { 'NULLS LAST' }
      let(:columns) { [{ order: field }] }
      let(:values) { [{ column: 0, dir: direction }] }

      it 'applies order' do
        expect(relation).to have_received(:order).with("#{field} #{direction} #{nulls}")
      end

      it 'returns relation' do
        expect(result).to eq(relation)
      end
    end

    context 'without direction' do
      let(:field) { 'field' }
      let(:columns) { [{ order: field }] }
      let(:values) { [{ column: 0 }] }

      it 'applies order with default direction' do
        expect(relation).to have_received(:order).with("#{field} ASC NULLS FIRST")
      end

      it 'returns relation' do
        expect(result).to eq(relation)
      end
    end

    context 'multiple' do
      let(:orders) do
        (0..1).map do |index|
          { field: "field#{index}", direction: 'DESC', nulls: 'NULLS LAST' }
        end
      end
      let(:columns) { orders.map { |order| { order: order[:field] } } }
      let(:values) do
        orders.map.with_index do |order, index|
          { column: index, dir: order[:direction] }
        end
      end

      it 'applies order' do
        orders.each do |order|
          expect(relation).to have_received(:order)
            .with("#{order[:field]} #{order[:direction]} #{order[:nulls]}")
        end
      end

      it 'returns relation' do
        expect(result).to eq(relation)
      end
    end
  end
end
