require 'spec_helper'

require 'tablets/data/processing/paginate'

RSpec.describe Tablets::Data::Processing::Paginate do
  subject { described_class.new(params, columns) }

  let(:relation) { double }
  let(:params) { { start: start, length: length } }
  let(:columns) { :columns }

  describe '#apply' do
    before do
      allow(relation).to receive(:offset).with(anything).and_return(relation)
      allow(relation).to receive(:limit).with(anything).and_return(relation)
    end

    let!(:result) { subject.apply(relation) }

    context 'if start matches beginning of page' do
      let(:start) { 30 }
      let(:length) { 15 }

      it 'applies start' do
        expect(relation).to have_received(:offset).with(start)
      end

      it 'applies length' do
        expect(relation).to have_received(:limit).with(length)
      end

      it 'returns relation' do
        expect(result).to eq(relation)
      end
    end

    context 'if start not matches beginning of page' do
      let(:start) { 25 }
      let(:page_start) { 15 }
      let(:length) { 15 }

      it 'applies page start' do
        expect(relation).to have_received(:offset).with(page_start)
      end

      it 'applies length' do
        expect(relation).to have_received(:limit).with(length)
      end

      it 'returns relation' do
        expect(result).to eq(relation)
      end
    end

    context 'without length' do
      let(:default_length) { 10 }
      let(:start) { 10 }
      let(:params) { { start: start } }

      it 'applies offset' do
        expect(relation).to have_received(:offset)
      end

      it 'applies default length' do
        expect(relation).to have_received(:limit).with(default_length)
      end

      it 'returns relation' do
        expect(result).to eq(relation)
      end
    end
  end
end
