require 'spec_helper'

require 'tablets'

require 'tablets/renderer'

RSpec.describe Tablets::Renderer do
  subject { described_class.new(tablet, params) }

  describe '#render' do
    let(:tablet) { double }
    let(:params) { { user_id: 1 } }

    let(:render_result) { 'result' }
    let(:path) { 'path' }

    let(:tablet_name) { :posts }
    let(:tablet_columns) do
      [{
        title: 'Title',
        order: 'posts.title',
        data: :title,
        options: { className: 'post-title' }
      }, {
        title: 'User',
        order: ['users.first_name', 'users.last_name'],
        data: :user,
        search: ['users.first_name', 'users.last_name']
      }]
    end
    let(:tablet_options) { { some_option: :some_value } }

    let(:options) do
      {
        ajax: path,
        columnDefs: [{
          targets: 0,
          searchable: false,
          orderable: true,
          className: 'post-title'
        }, {
          targets: 1,
          searchable: true,
          orderable: true
        }]
      }.merge!(tablet_options)
    end

    let(:view_context) { double }

    before do
      allow(tablet).to receive_messages(
        name: tablet_name,
        columns: tablet_columns,
        options: tablet_options,
        details?: false
      )

      allow(Tablets::Engine).to receive_message_chain(
        :routes, :url_helpers, :data_path
      ).and_return(path)

      allow(view_context).to receive(:render).and_return(render_result)
    end

    let!(:result) { subject.render(view_context) }

    def locals(local)
      hash_including(locals: hash_including(local))
    end

    it 'renders tablets partial' do
      expect(view_context).to have_received(:render)
        .with(hash_including(partial: 'tablets/tablet'))
    end

    it 'passes correct id as partial locals' do
      expect(view_context).to have_received(:render)
        .with(locals(id: 'posts_user_id_1_tablet'))
    end

    it 'passes correct columns as partial locals' do
      expect(view_context).to have_received(:render)
        .with(locals(columns: tablet_columns))
    end

    it 'passes correct config as partial locals' do
      expect(view_context).to have_received(:render)
        .with(locals(options: hash_including(options)))
    end

    it 'passes correct params as partial locals' do
      expect(view_context).to have_received(:render)
        .with(locals(params: params))
    end

    it 'returns render result' do
      expect(result).to eq(render_result)
    end
  end
end
