require 'spec_helper'

require 'tablets/utils/search_condition_builder'

RSpec.describe Tablets::Utils::SearchConditionBuilder do
  subject { described_class.new(column, query) }

  before do
    ActiveRecord::Base.establish_connection(adapter: 'sqlite3',
                                            database: ':memory:')

    allow(Rails).to receive(:env).and_return('env')
    allow(ActiveRecord::Base).to receive(:configurations)
      .and_return('env' => { 'adapter' => adapter })
  end

  describe '#build' do
    let(:column) { 'table.column' }
    let(:query) { 'query' }
    let!(:result) { subject.build }

    context 'postgresql' do
      let(:adapter) { 'postgresql' }

      it 'returns correct SQL statement' do
        expect(result).to eq(
          '(CAST ("table"."column" AS VARCHAR) ILIKE \'%query%\')'
        )
      end
    end

    context 'mysql2' do
      let(:adapter) { 'mysql2' }

      it 'returns correct SQL statement' do
        expect(result).to eq(
          '(CAST ("table"."column" AS CHAR) LIKE \'%query%\')'
        )
      end
    end

    context 'sqlite3' do
      let(:adapter) { 'sqlite3' }

      it 'returns correct SQL statement' do
        expect(result).to eq(
          '(CAST ("table"."column" AS TEXT) LIKE \'%query%\')'
        )
      end
    end
  end
end
