require 'spec_helper'

require 'tablets'

require 'tablets/view_helpers'

RSpec.describe Tablets::ViewHelpers do
  subject { Object.new.tap { |object| object.extend(described_class) } }

  describe '#render_tablet' do
    let(:name) { :name }
    let(:params) { {} }
    let(:initialize_param) { { initialize_instantly: false } }
    let(:tablet) { double }
    let(:renderer) { double }
    let(:render_result) { 'render_result' }

    before do
      allow(Tablets).to receive_message_chain(:[], :new).and_return(tablet)
      allow(Tablets::Renderer).to receive(:new)
        .with(tablet, params, initialize_param).and_return(renderer)
      allow(renderer).to receive(:render)
        .with(subject).and_return(render_result)
    end

    let!(:result) { subject.render_tablet(name, params) }

    it 'finds tablet' do
      expect(Tablets).to have_received(:[]).with(name.to_s)
    end

    it 'initializes renderer' do
      expect(Tablets::Renderer).to have_received(:new).with(tablet, params, initialize_param)
    end

    it 'renders in current view context' do
      expect(renderer).to have_received(:render).with(subject)
    end

    it 'returns render result' do
      expect(result).to eq(render_result)
    end
  end
end
